@@ -201,11 +201,7 @@ do_editor (dir, messagep, repository, changes)
     struct stat pre_stbuf, post_stbuf;
     int retcode = 0;
 
-#ifdef CLIENT_SUPPORT
     assert (!current_parsed_root->isremote != !repository);
-#else
-    assert (repository);
-#endif
 
     if (noexec || reuse_log_message)
 	return;
@@ -295,12 +291,7 @@ do_editor (dir, messagep, repository, changes)
     if (editinfo_editor)
 	free (editinfo_editor);
     editinfo_editor = (char *) NULL;
-#ifdef CLIENT_SUPPORT
-    if (current_parsed_root->isremote)
-	; /* nothing, leave editinfo_editor NULL */
-    else
-#endif
-    if (repository != NULL)
+    if (!current_parsed_root->isremote && repository != NULL)
 	(void) Parse_Info (CVSROOTADM_EDITINFO, repository, editinfo_proc, 0);
 
     /* run the editor */
@@ -427,11 +418,9 @@ do_verify (messagep, repository)
 
     struct stat pre_stbuf, post_stbuf;
 
-#ifdef CLIENT_SUPPORT
     if (current_parsed_root->isremote)
 	/* The verification will happen on the server.  */
 	return;
-#endif
 
     /* FIXME? Do we really want to skip this on noexec?  What do we do
        for the other administrative files?  */
