@@ -183,8 +183,7 @@ static const char *const usg[] =
        version control means.  */
 
     "For CVS updates and additional information, see\n",
-    "    the CVS home page at http://www.cvshome.org/ or\n",
-    "    Pascal Molli's CVS site at http://www.loria.fr/~molli/cvs-index.html\n",
+    "    the CVS home page at http://cvs.nongnu.org/\n",
     NULL,
 };
 
@@ -707,21 +706,18 @@ distribution kit for a complete list of contributors and copyrights.\n",
 	    cvs_cmd_name = "server";
 	}
 # endif /* AUTH_SERVER_SUPPORT || HAVE_GSSAPI */
+#endif /* SERVER_SUPPORT */
 
 	server_active = strcmp (cvs_cmd_name, "server") == 0;
 
-#endif /* SERVER_SUPPORT */
-
 	/* This is only used for writing into the history file.  For
 	   remote connections, it might be nice to have hostname
 	   and/or remote path, on the other hand I'm not sure whether
 	   it is worth the trouble.  */
 
-#ifdef SERVER_SUPPORT
 	if (server_active)
 	    CurDir = xstrdup ("<remote>");
 	else
-#endif
 	{
 	    CurDir = xgetwd ();
             if (CurDir == NULL)
@@ -780,13 +776,11 @@ distribution kit for a complete list of contributors and copyrights.\n",
 	if (use_cvsrc)
 	    read_cvsrc (&argc, &argv, cvs_cmd_name);
 
-#ifdef SERVER_SUPPORT
 	/* Fiddling with CVSROOT doesn't make sense if we're running
 	 * in server mode, since the client will send the repository
 	 * directory after the connection is made.
 	 */
 	if (!server_active)
-#endif
 	{
 	    /* First check if a root was set via the command line.  */
 	    if (CVSroot_cmdline)
@@ -873,20 +867,14 @@ distribution kit for a complete list of contributors and copyrights.\n",
 	   once).  To get out of the loop, we perform a "break" at the
 	   end of things.  */
 
-	while (
-#ifdef SERVER_SUPPORT
-	       server_active ||
-#endif
-	       walklist (root_directories, set_root_directory, NULL)
-	       )
+	while (server_active ||
+	       walklist (root_directories, set_root_directory, NULL))
 	{
-#ifdef SERVER_SUPPORT
 	    /* Fiddling with CVSROOT doesn't make sense if we're running
 	       in server mode, since the client will send the repository
 	       directory after the connection is made. */
 
 	    if (!server_active)
-#endif
 	    {
 		/* Now we're 100% sure that we have a valid CVSROOT
 		   variable.  Parse it to see if we're supposed to do
@@ -899,9 +887,7 @@ distribution kit for a complete list of contributors and copyrights.\n",
 		/*
 		 * Check to see if the repository exists.
 		 */
-#ifdef CLIENT_SUPPORT
 		if (!current_parsed_root->isremote)
-#endif	/* CLIENT_SUPPORT */
 		{
 		    char *path;
 		    int save_errno;
@@ -927,7 +913,6 @@ distribution kit for a complete list of contributors and copyrights.\n",
 		{
 		    static char *prev;
 		    char *env;
-		    size_t dummy;
 
 		    env = xmalloc (strlen (CVSROOT_ENV)
 				   + strlen (current_parsed_root->original)
@@ -951,14 +936,7 @@ distribution kit for a complete list of contributors and copyrights.\n",
 	       predetermine whether CVSROOT/config overrides things from
 	       read_cvsrc and other such places or vice versa.  That sort
 	       of thing probably needs more thought.  */
-	    if (1
-#ifdef SERVER_SUPPORT
-		&& !server_active
-#endif
-#ifdef CLIENT_SUPPORT
-		&& !current_parsed_root->isremote
-#endif
-		)
+	    if (!server_active && !current_parsed_root->isremote)
 	    {
 		/* If there was an error parsing the config file, parse_config
 		   already printed an error.  We keep going.  Why?  Because
@@ -988,9 +966,7 @@ distribution kit for a complete list of contributors and copyrights.\n",
                active, our list will be empty -- don't try and
                remove it from the list. */
 
-#ifdef SERVER_SUPPORT
 	    if (!server_active)
-#endif /* SERVER_SUPPORT */
 	    {
 		Node *n = findnode (root_directories,
 				    current_parsed_root->original);
@@ -1001,13 +977,11 @@ distribution kit for a complete list of contributors and copyrights.\n",
 		current_parsed_root = NULL;
 	    }
 
-#ifdef SERVER_SUPPORT
 	    if (server_active)
 	    {
 		server_active = 0;
 		break;
 	    }
-#endif
 	} /* end of loop for cvsroot values */
 
 	dellist (&root_directories);
