@@ -1,6 +1,11 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
  *
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -34,6 +39,7 @@
  */
 
 #include "cvs.h"
+#include <assert.h>
 #include "savecwd.h"
 #ifdef SERVER_SUPPORT
 # include "md5.h"
@@ -49,30 +55,29 @@ static int checkout_file PROTO ((struct file_info *finfo, Vers_TS *vers_ts,
 				 int adding, int merging, int update_server));
 #ifdef SERVER_SUPPORT
 static void checkout_to_buffer PROTO ((void *, const char *, size_t));
-#endif
-#ifdef SERVER_SUPPORT
 static int patch_file PROTO ((struct file_info *finfo,
 			      Vers_TS *vers_ts, 
 			      int *docheckout, struct stat *file_info,
 			      unsigned char *checksum));
 static void patch_file_write PROTO ((void *, const char *, size_t));
-#endif
+#endif /* SERVER_SUPPORT */
 static int merge_file PROTO ((struct file_info *finfo, Vers_TS *vers));
 static int scratch_file PROTO((struct file_info *finfo, Vers_TS *vers));
-static Dtype update_dirent_proc PROTO ((void *callerdat, char *dir,
-					char *repository, char *update_dir,
-					List *entries));
-static int update_dirleave_proc PROTO ((void *callerdat, char *dir,
-					int err, char *update_dir,
+static Dtype update_dirent_proc PROTO ((void *callerdat, const char *dir,
+                                        const char *repository,
+                                        const char *update_dir,
+                                        List *entries));
+static int update_dirleave_proc PROTO ((void *callerdat, const char *dir,
+					int err, const char *update_dir,
 					List *entries));
 static int update_fileproc PROTO ((void *callerdat, struct file_info *));
 static int update_filesdone_proc PROTO ((void *callerdat, int err,
-					 char *repository, char *update_dir,
-					 List *entries));
+                                         const char *repository,
+                                         const char *update_dir,
+                                         List *entries));
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
 static int get_linkinfo_proc PROTO ((void *callerdat, struct file_info *));
 #endif
-static void write_letter PROTO ((struct file_info *finfo, int letter));
 static void join_file PROTO ((struct file_info *finfo, Vers_TS *vers_ts));
 
 static char *options = NULL;
@@ -177,14 +182,12 @@ update (argc, argv)
 		break;
 	    case 'Q':
 	    case 'q':
-#ifdef SERVER_SUPPORT
 		/* The CVS 1.5 client sends these options (in addition to
 		   Global_option requests), so we must ignore them.  */
 		if (!server_active)
-#endif
 		    error (1, 0,
 			   "-q or -Q must be specified before \"%s\"",
-			   command_name);
+			   cvs_cmd_name);
 		break;
 	    case 'd':
 		update_build_dirs = 1;
@@ -196,6 +199,7 @@ update (argc, argv)
 		tag = optarg;
 		break;
 	    case 'D':
+		if (date) free (date);
 		date = Make_Date (optarg);
 		break;
 	    case 'P':
@@ -414,22 +418,25 @@ update (argc, argv)
     /* call the command line interface */
     err = do_update (argc, argv, options, tag, date, force_tag_match,
 		     local, update_build_dirs, aflag, update_prune_dirs,
-		     pipeout, which, join_rev1, join_rev2, (char *) NULL, 1);
+		     pipeout, which, join_rev1, join_rev2, (char *) NULL, 1,
+		     (char *) NULL);
 
     /* free the space Make_Date allocated if necessary */
     if (date != NULL)
 	free (date);
 
-    return (err);
+    return err;
 }
 
+
+
 /*
  * Command line interface to update (used by checkout)
  */
 int
 do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
 	   xprune, xpipeout, which, xjoin_rev1, xjoin_rev2, preload_update_dir,
-	   xdotemplate)
+	   xdotemplate, repository)
     int argc;
     char **argv;
     char *xoptions;
@@ -446,6 +453,7 @@ do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
     char *xjoin_rev2;
     char *preload_update_dir;
     int xdotemplate;
+    char *repository;
 {
     int err = 0;
     char *cp;
@@ -493,9 +501,9 @@ do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
 	err = start_recursion (get_linkinfo_proc, (FILESDONEPROC) NULL,
 			       (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
 			       argc, argv, local, which, aflag, CVS_LOCK_READ,
-			       preload_update_dir, 1);
+			       preload_update_dir, 1, (char *) NULL);
 	if (err)
-	    return (err);
+	    return err;
 
 	/* FIXME-twp: at this point we should walk the hardlist
 	   and update the `links' field of each hardlink_info struct
@@ -509,20 +517,15 @@ do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
     err = start_recursion (update_fileproc, update_filesdone_proc,
 			   update_dirent_proc, update_dirleave_proc, NULL,
 			   argc, argv, local, which, aflag, CVS_LOCK_READ,
-			   preload_update_dir, 1);
-
-#ifdef SERVER_SUPPORT
-    if (server_active)
-	return err;
-#endif
+			   preload_update_dir, 1, repository);
 
     /* see if we need to sleep before returning to avoid time-stamp races */
-    if (last_register_time)
+    if (!server_active && last_register_time)
     {
 	sleep_past (last_register_time);
     }
 
-    return (err);
+    return err;
 }
 
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
@@ -562,12 +565,14 @@ get_linkinfo_proc (callerdat, finfo)
     hlinfo->status = (Ctype) 0;	/* is this dumb? */
     hlinfo->checked_out = 0;
 
-    linkp->data = (char *) hlinfo;
+    linkp->data = hlinfo;
 
     return 0;
 }
 #endif
 
+
+
 /*
  * This is the callback proc for update.  It is called for each file in each
  * directory by the recursion code.  The current directory is the local
@@ -671,11 +676,7 @@ update_fileproc (callerdat, finfo)
                     bakname = backup_file (finfo->file, vers->vn_user);
                     /* This behavior is sufficiently unexpected to
                        justify overinformativeness, I think. */
-#ifdef SERVER_SUPPORT
-                    if ((! really_quiet) && (! server_active))
-#else /* ! SERVER_SUPPORT */
-                    if (! really_quiet)
-#endif /* SERVER_SUPPORT */
+                    if (!really_quiet && !server_active)
                         (void) printf ("(Locally modified %s moved to %s)\n",
                                        finfo->file, bakname);
                     free (bakname);
@@ -690,8 +691,7 @@ update_fileproc (callerdat, finfo)
                 {
                     if (vers->ts_conflict)
                     {
-			if ( file_has_conflict ( finfo, vers->ts_conflict )
-			     || file_has_markers ( finfo ) )
+			if (file_has_markers (finfo))
                         {
                             write_letter (finfo, 'C');
                             retval = 1;
@@ -706,10 +706,7 @@ update_fileproc (callerdat, finfo)
                         }
                     }
                     if (!retval)
-                    {
                         write_letter (finfo, 'M');
-                        retval = 0;
-                    }
                 }
 		break;
 	    case T_PATCH:		/* needs patch */
@@ -779,42 +776,48 @@ update_fileproc (callerdat, finfo)
     }
 
     freevers_ts (&vers);
-    return (retval);
+    return retval;
 }
 
-static void update_ignproc PROTO ((char *, char *));
+
+
+static void update_ignproc PROTO ((const char *, const char *));
 
 static void
 update_ignproc (file, dir)
-    char *file;
-    char *dir;
+    const char *file;
+    const char *dir;
 {
     struct file_info finfo;
+    char *tmp;
 
     memset (&finfo, 0, sizeof (finfo));
     finfo.file = file;
     finfo.update_dir = dir;
     if (dir[0] == '\0')
-	finfo.fullname = xstrdup (file);
+	tmp = xstrdup (file);
     else
     {
-	finfo.fullname = xmalloc (strlen (file) + strlen (dir) + 10);
-	strcpy (finfo.fullname, dir);
-	strcat (finfo.fullname, "/");
-	strcat (finfo.fullname, file);
+	tmp = xmalloc (strlen (file) + strlen (dir) + 10);
+	strcpy (tmp, dir);
+	strcat (tmp, "/");
+	strcat (tmp, file);
     }
 
+    finfo.fullname = tmp;
     write_letter (&finfo, '?');
-    free (finfo.fullname);
+    free (tmp);
 }
 
+
+
 /* ARGSUSED */
 static int
 update_filesdone_proc (callerdat, err, repository, update_dir, entries)
     void *callerdat;
     int err;
-    char *repository;
-    char *update_dir;
+    const char *repository;
+    const char *update_dir;
     List *entries;
 {
     if (rewrite_tag)
@@ -831,7 +834,7 @@ update_filesdone_proc (callerdat, err, repository, update_dir, entries)
     }
 
     /* Clean up CVS admin dirs if we are export */
-    if (strcmp (command_name, "export") == 0)
+    if (strcmp (cvs_cmd_name, "export") == 0)
     {
 	/* I'm not sure the existence_error is actually possible (except
 	   in cases where we really should print a message), but since
@@ -839,20 +842,18 @@ update_filesdone_proc (callerdat, err, repository, update_dir, entries)
 	if (unlink_file_dir (CVSADM) < 0 && !existence_error (errno))
 	    error (0, errno, "cannot remove %s directory", CVSADM);
     }
-#ifdef SERVER_SUPPORT
     else if (!server_active && !pipeout)
-#else
-    else if (!pipeout)
-#endif /* SERVER_SUPPORT */
     {
         /* If there is no CVS/Root file, add one */
         if (!isfile (CVSADM_ROOT))
 	    Create_Root ((char *) NULL, current_parsed_root->original);
     }
 
-    return (err);
+    return err;
 }
 
+
+
 /*
  * update_dirent_proc () is called back by the recursion processor before a
  * sub-directory is processed for update.  In this case, update_dirent proc
@@ -864,9 +865,9 @@ update_filesdone_proc (callerdat, err, repository, update_dir, entries)
 static Dtype
 update_dirent_proc (callerdat, dir, repository, update_dir, entries)
     void *callerdat;
-    char *dir;
-    char *repository;
-    char *update_dir;
+    const char *dir;
+    const char *repository;
+    const char *update_dir;
     List *entries;
 {
     if (ignore_directory (update_dir))
@@ -881,7 +882,7 @@ update_dirent_proc (callerdat, dir, repository, update_dir, entries)
     {
 	/* if we aren't building dirs, blow it off */
 	if (!update_build_dirs)
-	    return (R_SKIP_ALL);
+	    return R_SKIP_ALL;
 
 	/* Various CVS administrators are in the habit of removing
 	   the repository directory for things they don't want any
@@ -894,22 +895,18 @@ update_dirent_proc (callerdat, dir, repository, update_dir, entries)
 	   is when update -d is specified, and the working directory
 	   is gone but the subdirectory is still mentioned in
 	   CVS/Entries).  */
-	if (1
-#ifdef SERVER_SUPPORT
-	    /* In the remote case, the client should refrain from
-	       sending us the directory in the first place.  So we
-	       want to continue to give an error, so clients make
-	       sure to do this.  */
-	    && !server_active
-#endif
-	    && !isdir (repository))
+	/* In the remote case, the client should refrain from
+	   sending us the directory in the first place.  So we
+	   want to continue to give an error, so clients make
+	   sure to do this.  */
+	if (!server_active && !isdir (repository))
 	    return R_SKIP_ALL;
 
 	if (noexec)
 	{
 	    /* ignore the missing dir if -n is specified */
 	    error (0, 0, "New directory `%s' -- ignored", update_dir);
-	    return (R_SKIP_ALL);
+	    return R_SKIP_ALL;
 	}
 	else
 	{
@@ -1007,9 +1004,11 @@ update_dirent_proc (callerdat, dir, repository, update_dir, entries)
     if (!quiet)
 	error (0, 0, "Updating %s", update_dir);
 
-    return (R_PROCESS);
+    return R_PROCESS;
 }
 
+
+
 /*
  * update_dirleave_proc () is called back by the recursion code upon leaving
  * a directory.  It will prune empty directories if needed and will execute
@@ -1019,9 +1018,9 @@ update_dirent_proc (callerdat, dir, repository, update_dir, entries)
 static int
 update_dirleave_proc (callerdat, dir, err, update_dir, entries)
     void *callerdat;
-    char *dir;
+    const char *dir;
     int err;
-    char *update_dir;
+    const char *update_dir;
     List *entries;
 {
     /* Delete the ignore list if it hasn't already been done.  */
@@ -1065,9 +1064,11 @@ update_dirleave_proc (callerdat, dir, err, update_dir, entries)
 	}
     }
 
-    return (err);
+    return err;
 }
 
+
+
 static int isremoved PROTO ((Node *, void *));
 
 /* Returns 1 if the file indicated by node has been removed.  */
@@ -1076,19 +1077,21 @@ isremoved (node, closure)
     Node *node;
     void *closure;
 {
-    Entnode *entdata = (Entnode*) node->data;
+    Entnode *entdata = node->data;
 
     /* If the first character of the version is a '-', the file has been
        removed. */
     return (entdata->version && entdata->version[0] == '-') ? 1 : 0;
 }
 
+
+
 /* Returns 1 if the argument directory is completely empty, other than the
    existence of the CVS directory entry.  Zero otherwise.  If MIGHT_NOT_EXIST
    and the directory doesn't exist, then just return 0.  */
 int
 isemptydir (dir, might_not_exist)
-    char *dir;
+    const char *dir;
     int might_not_exist;
 {
     DIR *dirp;
@@ -1099,7 +1102,7 @@ isemptydir (dir, might_not_exist)
 	if (might_not_exist && existence_error (errno))
 	    return 0;
 	error (0, errno, "cannot open directory %s for empty check", dir);
-	return (0);
+	return 0;
     }
     errno = 0;
     while ((dp = CVS_READDIR (dirp)) != NULL)
@@ -1112,7 +1115,7 @@ isemptydir (dir, might_not_exist)
 		/* An entry other than the CVS directory.  The directory
 		   is certainly not empty. */
 		(void) CVS_CLOSEDIR (dirp);
-		return (0);
+		return 0;
 	    }
 	    else
 	    {
@@ -1143,7 +1146,7 @@ isemptydir (dir, might_not_exist)
 		    /* There are files that have been removed, but not
 		       committed!  Do not consider the directory empty. */
 		    (void) CVS_CLOSEDIR (dirp);
-		    return (0);
+		    return 0;
 		}
 	    }
 	}
@@ -1153,12 +1156,14 @@ isemptydir (dir, might_not_exist)
     {
 	error (0, errno, "cannot read directory %s", dir);
 	(void) CVS_CLOSEDIR (dirp);
-	return (0);
+	return 0;
     }
     (void) CVS_CLOSEDIR (dirp);
-    return (1);
+    return 1;
 }
 
+
+
 /*
  * scratch the Entries file entry associated with a file
  */
@@ -1182,13 +1187,10 @@ scratch_file (finfo, vers)
 #endif
     if (unlink_file (finfo->file) < 0 && ! existence_error (errno))
 	error (0, errno, "unable to remove %s", finfo->fullname);
-    else
-#ifdef SERVER_SUPPORT
+    else if (!server_active)
+    {
 	/* skip this step when the server is running since
 	 * server_updated should have handled it */
-	if (!server_active)
-#endif
-    {
 	/* keep the vers structure up to date in case we do a join
 	 * - if there isn't a file, it can't very well have a version number, can it?
 	 */
@@ -1203,9 +1205,11 @@ scratch_file (finfo, vers)
 	    vers->ts_user = NULL;
 	}
     }
-    return (0);
+    return 0;
 }
 
+
+
 /*
  * Check out a file.
  */
@@ -1228,11 +1232,7 @@ checkout_file (finfo, vers_ts, adding, merging, update_server)
 
     /* Don't screw with backup files if we're going to stdout, or if
        we are the server.  */
-    if (!pipeout
-#ifdef SERVER_SUPPORT
-	&& ! server_active
-#endif
-	)
+    if (!pipeout && !server_active)
     {
 	backup = xmalloc (strlen (finfo->file)
 			  + sizeof (CVSADM)
@@ -1323,8 +1323,13 @@ VERS: ", 0);
 		   is here only because noexec doesn't write srcfile->path
 		   for us to stat.  */
 		if (stat (vers_ts->srcfile->path, &sb) < 0)
+		{
+#if defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT)
+		    buf_free (revbuf);
+#endif /* defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT) */
 		    error (1, errno, "cannot stat %s",
 			   vers_ts->srcfile->path);
+		}
 		mode = sb.st_mode &~ (S_IWRITE | S_IWGRP | S_IWOTH);
 	    }
 
@@ -1430,6 +1435,8 @@ VERS: ", 0);
 	    /* fix up the vers structure, in case it is used by join */
 	    if (join_rev1)
 	    {
+		/* FIXME: Throwing away the original revision info is almost
+		   certainly wrong -- what if join_rev1 is "BASE"?  */
 		if (vers_ts->vn_user != NULL)
 		    free (vers_ts->vn_user);
 		if (vers_ts->vn_rcs != NULL)
@@ -1439,7 +1446,7 @@ VERS: ", 0);
 	    }
 
 	    /* If this is really Update and not Checkout, recode history */
-	    if (strcmp (command_name, "update") == 0)
+	    if (strcmp (cvs_cmd_name, "update") == 0)
 		history_write ('U', finfo->update_dir, xvers_ts->vn_rcs, finfo->file,
 			       finfo->repository);
 
@@ -1486,9 +1493,15 @@ VERS: ", 0);
 	free (backup);
     }
 
-    return (retval);
+#if defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT)
+    if (revbuf != NULL)
+	buf_free (revbuf);
+#endif /* defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT) */
+    return retval;
 }
 
+
+
 #ifdef SERVER_SUPPORT
 
 /* This function is used to write data from a file being checked out
@@ -1684,7 +1697,9 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
     retcode = 0;
     if (! fail)
     {
-	char *diff_options;
+	int dargc = 0;
+	size_t darg_allocated = 0;
+	char **dargv = NULL;
 
 	/* If the client does not support the Rcs-diff command, we
            send a context diff, and the client must invoke patch.
@@ -1692,16 +1707,13 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
            new approach only requires running diff in the server; the
            client can handle everything without invoking an external
            program.  */
-	if (! rcs_diff_patches)
-	{
+	if (!rcs_diff_patches)
 	    /* We use -c, not -u, because that is what CVS has
 	       traditionally used.  Kind of a moot point, now that
 	       Rcs-diff is preferred, so there is no point in making
 	       the compatibility issues worse.  */
-	    diff_options = "-c";
-	}
+	    run_add_arg_p (&dargc, &darg_allocated, &dargv, "-c");
 	else
-	{
 	    /* Now that diff is librarified, we could be passing -a if
 	       we wanted to.  However, it is unclear to me whether we
 	       would want to.  Does diff -a, in any significant
@@ -1711,10 +1723,11 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
 	       'binary'.  Conversely, do they tend to be much larger
 	       in the bad cases?  This needs some more
 	       thought/investigation, I suspect.  */
-
-	    diff_options = "-n";
-	}
-	retcode = diff_exec (file1, file2, NULL, NULL, diff_options, finfo->file);
+	    run_add_arg_p (&dargc, &darg_allocated, &dargv, "-n");
+	retcode = diff_exec (file1, file2, NULL, NULL, dargc, dargv,
+			     finfo->file);
+	run_arg_free_p (dargc, dargv);
+	free (dargv);
 
 	/* A retcode of 0 means no differences.  1 means some differences.  */
 	if (retcode != 0
@@ -1790,10 +1803,10 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
 	if (CVS_STAT (finfo->file, file_info) < 0)
 	    error (1, errno, "could not stat %s", finfo->file);
 
-	/* If this is really Update and not Checkout, recode history */
-	if (strcmp (command_name, "update") == 0)
-	    history_write ('P', finfo->update_dir, xvers_ts->vn_rcs, finfo->file,
-			   finfo->repository);
+	/* If this is really Update and not Checkout, record history.  */
+	if (strcmp (cvs_cmd_name, "update") == 0)
+	    history_write ('P', finfo->update_dir, xvers_ts->vn_rcs,
+	                   finfo->file, finfo->repository);
 
 	freevers_ts (&xvers_ts);
 
@@ -1830,9 +1843,11 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
     free (backup);
     free (file1);
     free (file2);
-    return (retval);
+    return retval;
 }
 
+
+
 /* Write data to a file.  Record whether the last byte written was a
    newline.  Optionally compute a checksum.  This is called by
    patch_file via RCS_checkout.  */
@@ -1860,7 +1875,7 @@ patch_file_write (callerdat, buffer, len)
  * Several of the types we process only print a bit of information consisting
  * of a single letter and the name.
  */
-static void
+void
 write_letter (finfo, letter)
     struct file_info *finfo;
     int letter;
@@ -1901,6 +1916,49 @@ write_letter (finfo, letter)
     return;
 }
 
+
+
+/* Reregister a file after a merge.  */
+static void
+RegisterMerge PROTO((struct file_info *finfo, Vers_TS *vers,
+		     const char *backup, int has_conflicts));
+void
+RegisterMerge (finfo, vers, backup, has_conflicts)
+    struct file_info *finfo;
+    Vers_TS *vers;
+    const char *backup;
+    int has_conflicts;
+{
+    /* This file is the result of a merge, which means that it has
+       been modified.  We use a special timestamp string which will
+       not compare equal to any actual timestamp.  */
+    char *cp = NULL;
+
+    if (has_conflicts)
+    {
+	time (&last_register_time);
+	cp = time_stamp (finfo->file);
+    }
+    Register (finfo->entries, finfo->file, vers->vn_rcs ? vers->vn_rcs : "0",
+	      "Result of merge", vers->options, vers->tag, vers->date, cp);
+    if (cp)
+	free (cp);
+
+#ifdef SERVER_SUPPORT
+    /* Send the new contents of the file before the message.  If we
+       wanted to be totally correct, we would have the client write
+       the message only after the file has safely been written.  */
+    if (server_active)
+    {
+        server_copy_file (finfo->file, finfo->update_dir, finfo->repository,
+			  backup);
+	server_updated (finfo, vers, SERVER_MERGED, (mode_t) -1, NULL, NULL);
+    }
+#endif
+}
+
+
+
 /*
  * Do all the magic associated with a file which needs to be merged
  */
@@ -1914,6 +1972,8 @@ merge_file (finfo, vers)
     int retcode = 0;
     int retval;
 
+    assert (vers->vn_user);
+
     /*
      * The users currently modified file is moved to a backup file name
      * ".#filename.version", so that it will stay around for a few days
@@ -1951,13 +2011,21 @@ merge_file (finfo, vers)
 	   thought needs to go into this, and in the meantime it is safe
 	   to treat any such mismatch as an automatic conflict. -twp */
 
-#ifdef SERVER_SUPPORT
-	if (server_active)
-	    server_copy_file (finfo->file, finfo->update_dir,
-			      finfo->repository, backup);
-#endif
+	retcode = RCS_checkout (finfo->rcs, finfo->file,
+				vers->vn_rcs, vers->tag,
+				vers->options, NULL, NULL, NULL);
+	if (retcode)
+	{
+	    error (0, 0, "failed to check out `%s' file", finfo->fullname);
+	    error (0, 0, "restoring `%s' from backup file `%s'",
+		   finfo->fullname, backup);
+	    rename_file (backup, finfo->file);
+	    retval = 1;
+	    goto out;
+	}
+	xchmod (finfo->file, 1);
 
-	status = checkout_file (finfo, vers, 0, 1, 1);
+	RegisterMerge (finfo, vers, backup, 1);
 
 	/* Is there a better term than "nonmergeable file"?  What we
 	   really mean is, not something that CVS cannot or does not
@@ -1975,8 +2043,8 @@ merge_file (finfo, vers)
 	goto out;
     }
 
-    status = RCS_merge(finfo->rcs, vers->srcfile->path, finfo->file,
-		       vers->options, vers->vn_user, vers->vn_rcs);
+    status = RCS_merge (finfo->rcs, vers->srcfile->path, finfo->file,
+		        vers->options, vers->vn_user, vers->vn_rcs);
     if (status != 0 && status != 1)
     {
 	error (0, status == -1 ? errno : 0,
@@ -1991,45 +2059,17 @@ merge_file (finfo, vers)
     if (strcmp (vers->options, "-V4") == 0)
 	vers->options[0] = '\0';
 
-    /* This file is the result of a merge, which means that it has
-       been modified.  We use a special timestamp string which will
-       not compare equal to any actual timestamp.  */
-    {
-	char *cp = 0;
-
-	if (status)
-	{
-	    (void) time (&last_register_time);
-	    cp = time_stamp (finfo->file);
-	}
-	Register (finfo->entries, finfo->file, vers->vn_rcs,
-		  "Result of merge", vers->options, vers->tag,
-		  vers->date, cp);
-	if (cp)
-	    free (cp);
-    }
-
     /* fix up the vers structure, in case it is used by join */
     if (join_rev1)
     {
+	/* FIXME: Throwing away the original revision info is almost
+	   certainly wrong -- what if join_rev1 is "BASE"?  */
 	if (vers->vn_user != NULL)
 	    free (vers->vn_user);
 	vers->vn_user = xstrdup (vers->vn_rcs);
     }
 
-#ifdef SERVER_SUPPORT
-    /* Send the new contents of the file before the message.  If we
-       wanted to be totally correct, we would have the client write
-       the message only after the file has safely been written.  */
-    if (server_active)
-    {
-        server_copy_file (finfo->file, finfo->update_dir, finfo->repository,
-			  backup);
-	server_updated (finfo, vers, SERVER_MERGED,
-			(mode_t) -1, (unsigned char *) NULL,
-			(struct buffer *) NULL);
-    }
-#endif
+    RegisterMerge (finfo, vers, backup, status);
 
     /* FIXME: the noexec case is broken.  RCS_merge could be doing the
        xcmp on the temporary files without much hassle, I think.  */
@@ -2054,7 +2094,8 @@ merge_file (finfo, vers)
 
 	write_letter (finfo, 'C');
 
-	history_write ('C', finfo->update_dir, vers->vn_rcs, finfo->file, finfo->repository);
+	history_write ('C', finfo->update_dir, vers->vn_rcs, finfo->file,
+	               finfo->repository);
 
     }
     else if (retcode == -1)
@@ -2074,9 +2115,23 @@ merge_file (finfo, vers)
     return retval;
 }
 
+
+
 /*
  * Do all the magic associated with a file which needs to be joined
- * (-j option)
+ * (reached via the -j option to checkout or update).
+ *
+ * INPUTS
+ *   finfo		File information about the destination file.
+ *   vers		The Vers_TS structure for finfo.
+ *
+ * GLOBALS
+ *   join_rev1		From the command line.
+ *   join_rev2		From the command line.
+ *   server_active	Natch.
+ *
+ * ASSUMPTIONS
+ *   1.  Is not called in client mode.
  */
 static void
 join_file (finfo, vers)
@@ -2127,6 +2182,9 @@ join_file (finfo, vers)
 	jdate1 = NULL;
     }
 
+    /* FIXME: Need to handle "BASE" for jrev1 and/or jrev2.  Note caveat
+       below about vn_user.  */
+
     /* Convert the second revision, walking branches and dates.  */
     rev2 = RCS_getversion (vers->srcfile, jrev2, jdate2, 1, (int *) NULL);
 
@@ -2308,13 +2366,41 @@ join_file (finfo, vers)
 	return;
     }
 
-    /* If the target of the merge is the same as the working file
-       revision, then there is nothing to do.  */
-    if (vers->vn_user != NULL && strcmp (rev2, vers->vn_user) == 0)
+    /* If the two merge revisions are the same, then there is nothing
+     * to do.  This needs to be checked before the rev2 == up-to-date base
+     * revision check tha comes next.  Otherwise, rev1 can == rev2 and get an
+     * "already contains the changes between <rev1> and <rev1>" message.
+     */
+    if (rev1 && strcmp (rev1, rev2) == 0)
     {
+	free (rev1);
+	free (rev2);
+	return;
+    }
+
+    /* If we know that the user file is up-to-date, then it becomes an
+     * optimization to skip the merge when rev2 is the same as the base
+     * revision.  i.e. we know that diff3(file2,file1,file2) will produce
+     * file2.
+     */
+    if (vers->vn_user != NULL && vers->ts_user != NULL
+        && strcmp (vers->ts_user, vers->ts_rcs) == 0
+        && strcmp (rev2, vers->vn_user) == 0)
+    {
+	if (!really_quiet)
+	{
+	    cvs_output (finfo->fullname, 0);
+	    cvs_output (" already contains the differences between ", 0);
+	    cvs_output (rev1 ? rev1 : "creation", 0);
+	    cvs_output (" and ", 0);
+	    cvs_output (rev2, 0);
+	    cvs_output ("\n", 1);
+	}
+
 	if (rev1 != NULL)
 	    free (rev1);
 	free (rev2);
+
 	return;
     }
 
@@ -2369,17 +2455,8 @@ join_file (finfo, vers)
 	return;
     }
 
-    /* If the two merge revisions are the same, then there is nothing
-       to do.  */
-    if (strcmp (rev1, rev2) == 0)
-    {
-	free (rev1);
-	free (rev2);
-	return;
-    }
-
     /* If there is no working file, then we can't do the merge.  */
-    if (vers->vn_user == NULL)
+    if (vers->vn_user == NULL || vers->vn_user[0] == '-')
     {
 	free (rev1);
 	free (rev2);
@@ -2542,16 +2619,39 @@ join_file (finfo, vers)
 	status = RCS_merge (finfo->rcs, vers->srcfile->path, finfo->file,
 			    t_options, rev1, rev2);
 
-    if (status != 0 && status != 1)
+    if (status != 0)
     {
-	error (0, status == -1 ? errno : 0,
-	       "could not merge revision %s of %s", rev2, finfo->fullname);
-	error (status == -1 ? 1 : 0, 0, "restoring %s from backup file %s",
-	       finfo->fullname, backup);
-	rename_file (backup, finfo->file);
+	if (status != 1)
+	{
+	    error (0, status == -1 ? errno : 0,
+		   "could not merge revision %s of %s", rev2, finfo->fullname);
+	    error (status == -1 ? 1 : 0, 0, "restoring %s from backup file %s",
+		   finfo->fullname, backup);
+	    rename_file (backup, finfo->file);
+	}
+    }
+    else /* status == 0 */
+    {
+	/* FIXME: the noexec case is broken.  RCS_merge could be doing the
+	   xcmp on the temporary files without much hassle, I think.  */
+	if (!noexec && !xcmp (backup, finfo->file))
+	{
+	    if (!really_quiet)
+	    {
+		cvs_output (finfo->fullname, 0);
+		cvs_output (" already contains the differences between ", 0);
+		cvs_output (rev1, 0);
+		cvs_output (" and ", 0);
+		cvs_output (rev2, 0);
+		cvs_output ("\n", 1);
+	    }
+
+	    /* and skip the registering and sending the new file since it
+	     * hasn't been updated.
+	     */
+	    goto out;
+	}
     }
-    free (rev1);
-    free (rev2);
 
     /* The file has changed, but if we just checked it out it may
        still have the same timestamp it did when it was first
@@ -2563,34 +2663,16 @@ join_file (finfo, vers)
        RCS_checkout above, and we aren't running as the server.
        However, that is not the normal case, and calling Register
        again won't cost much in that case.  */
-    {
-	char *cp = 0;
+    RegisterMerge (finfo, vers, backup, status);
 
-	if (status)
-	{
-	    (void) time (&last_register_time);
-	    cp = time_stamp (finfo->file);
-	}
-	Register (finfo->entries, finfo->file,
-		  vers->vn_rcs ? vers->vn_rcs : "0", "Result of merge",
-		  vers->options, vers->tag, vers->date, cp);
-	if (cp)
-	    free(cp);
-    }
-
-#ifdef SERVER_SUPPORT
-    if (server_active)
-    {
-	server_copy_file (finfo->file, finfo->update_dir, finfo->repository,
-			  backup);
-	server_updated (finfo, vers, SERVER_MERGED,
-			(mode_t) -1, (unsigned char *) NULL,
-			(struct buffer *) NULL);
-    }
-#endif
+out:
+    free (rev1);
+    free (rev2);
     free (backup);
 }
 
+
+
 /*
  * Report whether revisions REV1 and REV2 of FINFO agree on:
  *   . file ownership
@@ -2670,7 +2752,7 @@ special_file_mismatch (finfo, rev1, rev2)
     else
     {
 	n = findnode (finfo->rcs->versions, rev1);
-	vp = (RCSVers *) n->data;
+	vp = n->data;
 
 	n = findnode (vp->other_delta, "symlink");
 	if (n != NULL)
@@ -2705,7 +2787,7 @@ special_file_mismatch (finfo, rev1, rev2)
 		if (sscanf (n->data, "%15s %lu", ftype,
 			    &dev_long) < 2)
 		    error (1, 0, "%s:%s has bad `special' newphrase %s",
-			   finfo->file, rev1, n->data);
+			   finfo->file, rev1, (char *)n->data);
 		rev1_dev = dev_long;
 		if (strcmp (ftype, "character") == 0)
 		    rev1_mode |= S_IFCHR;
@@ -2748,7 +2830,7 @@ special_file_mismatch (finfo, rev1, rev2)
     else
     {
 	n = findnode (finfo->rcs->versions, rev2);
-	vp = (RCSVers *) n->data;
+	vp = n->data;
 
 	n = findnode (vp->other_delta, "symlink");
 	if (n != NULL)
@@ -2783,7 +2865,7 @@ special_file_mismatch (finfo, rev1, rev2)
 		if (sscanf (n->data, "%15s %lu", ftype,
 			    &dev_long) < 2)
 		    error (1, 0, "%s:%s has bad `special' newphrase %s",
-			   finfo->file, rev2, n->data);
+			   finfo->file, rev2, (char *)n->data);
 		rev2_dev = dev_long;
 		if (strcmp (ftype, "character") == 0)
 		    rev2_mode |= S_IFCHR;
@@ -2902,10 +2984,10 @@ special_file_mismatch (finfo, rev1, rev2)
 #endif
 }
 
+
+
 int
 joining ()
 {
-    return (join_rev1 != NULL);
+    return join_rev1 != NULL;
 }
-/* vim:tabstop=8:shiftwidth=4
- */
